#include "stdafx.h"
#include "InputHandler.h"

using namespace Ogre;

InputHandler::InputHandler(Ogre::RenderWindow* renderWindow)
{
	mTimeUntilNextToggle = -1;
	Ogre::LogManager::getSingletonPtr()->logMessage("*** Initializing OIS ***");

	OIS::ParamList pl;

	size_t windowHnd = 0;
	renderWindow->getCustomAttribute("WINDOW", &windowHnd);
	std::ostringstream windowHndStr;
	windowHndStr << windowHnd;
	pl.insert(std::make_pair(std::string("WINDOW"), windowHndStr.str()));
	OISInputManager = OIS::InputManager::createInputSystem( pl );

	mKeyboard = static_cast<OIS::Keyboard*>(OISInputManager->createInputObject( OIS::OISKeyboard, true ));
	mMouse = static_cast<OIS::Mouse*>(OISInputManager->createInputObject( OIS::OISMouse, true ));

	mKeyboard->setEventCallback(this);
	mMouse->setEventCallback(this);
}

InputHandler::~InputHandler(void)
{
	Ogre::LogManager::getSingletonPtr()->logMessage("*** Shutting down OIS ***");
	OIS::InputManager::destroyInputSystem(OISInputManager);
}

void InputHandler::setPlayerController(PlayerController* c)
{
	playerController = c;
}

bool InputHandler::update(float t, float dt)
{
	mKeyboard->capture();
	mMouse->capture();

	const OIS::MouseState &ms = mMouse->getMouseState();

	if (mTimeUntilNextToggle >= 0)
		mTimeUntilNextToggle -= dt;

	if( mKeyboard->isKeyDown(OIS::KC_ESCAPE))
		exit(0);

	playerController->update(t, dt, mKeyboard, mMouse);
	return true;
}

bool InputHandler::keyPressed(const OIS::KeyEvent &keyEventRef)
{
	return playerController->keyPressed(keyEventRef);
}

bool InputHandler::keyReleased(const OIS::KeyEvent &keyEventRef) {return true;}

bool InputHandler::mouseMoved(const OIS::MouseEvent &evt) {
	return playerController->mouseMoved(evt);
}
bool InputHandler::mousePressed(const OIS::MouseEvent &evt, OIS::MouseButtonID id) {
	return playerController->mousePressed(evt, id);
}
bool InputHandler::mouseReleased(const OIS::MouseEvent &evt, OIS::MouseButtonID id) {
	return playerController->mouseReleased(evt, id);
}



