#include "stdafx.h"
#include "Level.h"
//#include "Truck.h"
#include "Crate.h"
#include "Target.h"
#include "AbstractTarget.h"
//#include <fstream>

//using namespace std;
using namespace Ogre;

Level::Level(SceneManager* sm)
{
	SceneNode* rootNode = sm->getRootSceneNode();
	truck = new Truck(sm);

	Plane plane(Vector3::UNIT_Y, 0);
	MeshManager::getSingleton().createPlane("ground",ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, plane,
			                                6e3,6e3,1,1,true,1,1,1,Vector3::UNIT_Z);
	Entity* planeEntity = sm->createEntity("GroundEntity", "ground");
    sm->getRootSceneNode()->createChildSceneNode()->attachObject(planeEntity);
    planeEntity->setMaterialName("Plane");
    planeEntity->setCastShadows(false);

	sm->setSkyBox(true, "Sky", 10, true);	

	//targetList.push_back( new Target(sm, 30, 20, 20) );

	//targetList.push_back( new Crate(sm, -30, 8, -20) );

	//targetList.push_back( new Crate(sm, 30, 8, -20) );
	//targetList.push_back( new Crate(sm, 30, 22.8f, -20) );

	const int numCrates = 35;
	const int numTargets = 35;

	for(int i=0; i<numCrates; i++)
	{
		float x = Math::RangeRandom(-3000, 3000);
		float z = Math::RangeRandom(-1000, 1000);
		Vector3 pos(x,8,z);
		if (isFreePosInside(pos)) targetList.push_back( new Crate(sm, pos.x, pos.y, pos.z) );
		else i--;
	}

	for(int i=0; i<numTargets; i++)
	{
		float x = Math::RangeRandom(-3000, 3000);
		float z = Math::RangeRandom(-1000, 1000);
		Vector3 pos(x,20,z);
		if (isFreePosOutside(pos)) targetList.push_back( new Target(sm, pos.x, pos.y, pos.z) );
		else i--;
	}

}

Level::~Level(void)
{
}
bool Level::isFreePosInside(Ogre::Vector3 pos)
{
	//ellipse parameters
	const float a = 2300;
	const float b = 450;
	bool ret = true;
	std::deque<AbstractTarget*>::iterator it;
	for( it = targetList.begin(); it != targetList.end(); ++it ) 
	{
		float dist = pos.distance((*it)->getPosition());
		if (dist < (*it)->getRadius()) ret = false;
		float ell = pos.x*pos.x/a/a + pos.z*pos.z/b/b - 1;
		if (ell > 0) ret = false;
	}
	return ret;
}

bool Level::isFreePosOutside(Ogre::Vector3 pos)
{
	//ellipse parameters
	const float a = 2700;
	const float b = 550;
	bool ret = true;
	std::deque<AbstractTarget*>::iterator it;
	for( it = targetList.begin(); it != targetList.end(); ++it ) 
	{
		float dist = pos.distance((*it)->getPosition());
		if (dist < (*it)->getRadius()) ret = false;
		float ell = pos.x*pos.x/a/a + pos.z*pos.z/b/b - 1;
		if (ell < 0) ret = false;
	}
	return ret;
}

float Level::shootTargetAt(Ogre::Vector3 hitPos)
{
	float minDist = 1e10;
	AbstractTarget* ret = 0;
	std::deque<AbstractTarget*>::iterator it;
	for( it = targetList.begin(); it != targetList.end(); ++it ) 
	{
		float dist = hitPos.distance((*it)->getPosition());
		//printf("dist=%f; ",dist);
		if (dist < minDist) 
		{
			minDist=dist;
			ret = *it;
		}
	}
	//printf("\n");
	if (minDist == 1e10) return 0;
	if (ret) 
	{
		if (ret->getRadius() > minDist) 
		{
			return ret->reduceHealth(minDist);
			//return ret;
		}
		else return 0;
	}
	return 0;
}










