#include "stdafx.h"
#include "PlayerCharacter.h"
#include "Hud.h"
#include "Level.h"

using namespace Ogre;

PlayerCharacter::PlayerCharacter(Ogre::SceneManager* sm, Hud* h, Level* lv)
{
	hud = h;
	level = lv;
	truckNode = 0;
	followTruck = false;
	entity = sm->createEntity("player", "soldier.mesh");
	node = sm->getRootSceneNode()->createChildSceneNode();
	node->attachObject(entity);
	//node->translate(0,-50,0);
	//entity->setCastShadows(true);

	entity->getSkeleton()->setBlendMode(ANIMBLEND_CUMULATIVE);
	poseAnim = entity->getAnimationState("leg_stand");
	actionAnim = entity->getAnimationState("up_weapon_hold");
	poseAnim->setEnabled(true);
	actionAnim->setEnabled(true);
	poseAnim->setWeight(1);
	actionAnim->setWeight(1);
	poseAnim->setLoop(true);
	actionAnim->setLoop(true);

	lastAction = action = PA_WEAPON_HOLD;
	lastPose = pose = PP_STAND;
	shootState = SH_TAKE_AIM;

//	hasWeapon = true;

	Entity* weapon = sm->createEntity("weapon", "rifle.mesh");
	weaponNode = node->createChildSceneNode();
	SceneNode* tempNode = weaponNode->createChildSceneNode();
	tempNode->rotate(Vector3::UNIT_X, Radian(Degree(90)));
	tempNode->attachObject(weapon);

	lookDir = node->getOrientation() * Ogre::Vector3::UNIT_Z;
	raySceneQuery = sm->createRayQuery( Ray() );
	raySceneQuery->setSortByDistance(true, 4);
}

PlayerCharacter::~PlayerCharacter(void)
{
}

/*
void PlayerCharacter::setParentNode(Ogre::SceneNode* parent)
{ 
	node->detachAllObjects();
	node=parent;
	node->attachObject(entity);
}
*/

void PlayerCharacter::setFollowTruck(bool b)
{
	if ( b == true && truckNode != 0) followTruck = true;
	else 
	{
		followTruck = false;
		Vector3 v = truckNode->getPosition() + truckNode->getOrientation() * Vector3(0,0,15);
		node->setPosition(v.x, 0, v.z);
	}
}

void PlayerCharacter::update(float t, float dt)
{	
	node->setDirection( -lookDir * Vector3(1,0,1), Ogre::Node::TransformSpace::TS_WORLD );
	if (followTruck) node->setPosition( truckNode->getPosition() + truckNode->getOrientation() * Vector3(0,0,15) );
	//hud->hits = action;
	if(action != lastAction || pose != lastPose)
	{
		if(!poseAnim->getLoop() && poseAnim->hasEnded())
			pose = PP_STAND;
		if(!actionAnim->getLoop() && actionAnim->hasEnded())
			action = PA_WEAPON_HOLD;
		
		poseAnim->setEnabled(false);
		actionAnim->setEnabled(false);
		poseAnim = 0;
		actionAnim = 0;
		
		if(action == PA_WEAPON_HOLD)
		{
			actionAnim = entity->getAnimationState("up_weapon_hold");
			actionAnim->setLoop(true);
		}
		else if(action == PA_SHOOT)
		{
			actionAnim = entity->getAnimationState("up_shoot");
			actionAnim->setTimePosition(0);
			actionAnim->setLoop(false);
		}

		if(pose == PP_STAND)
		{
			poseAnim = entity->getAnimationState("leg_stand");
			poseAnim->setLoop(true);
		}
		else if(pose == PP_RUN)
		{
			poseAnim = entity->getAnimationState("leg_run");
			poseAnim->setLoop(true);
		}

		if(poseAnim)
			poseAnim->setEnabled(true);
		if(actionAnim)
			actionAnim->setEnabled(true);

		lastAction = action;
		lastPose = pose;
	}
	const float shStart = 0.75f;
	const float shEnd =0.80f;
	
	if (action == PA_SHOOT)
	{
		if (shootState == SH_TAKE_AIM)
		{
			if (actionAnim->getTimePosition() < shStart) actionAnim->addTime(dt);
			else shootState = SH_AIM;
		}
		else if (shootState == SH_AIM)
		{
			if (actionAnim->getTimePosition() != shStart && actionAnim->getTimePosition() < shEnd) 
				actionAnim->addTime(dt);
			else actionAnim->setTimePosition(shStart);

		}
		else if (shootState == SH_SHOOT)
		{
			if (actionAnim->getTimePosition() > shEnd) 
			{
				if (hud->magazine > 0) 
				{
					hud->magazine--;
					hitQuery();
					actionAnim->setTimePosition(shStart);
				}
				
			}else actionAnim->addTime(dt / 2);
		}
		else if (shootState == SH_END_AIM)
		{
			if (actionAnim->getTimePosition() < 1.4f) actionAnim->setTimePosition(1.4f);
			actionAnim->addTime(dt);
		}
		else if (shootState == SH_RELOAD)
		{
			if (fabs(actionAnim->getTimePosition()-0.5f) < 0.1f) actionAnim->setTimePosition(1.5f);
			actionAnim->addTime(dt/2);
			if (actionAnim->getTimePosition() >= 1.8f) hud->setShowMagazineCounter(true);
		}
		
	}
	else actionAnim->addTime(dt);
	poseAnim->addTime(dt);

	if(!inAction())
		action = PA_WEAPON_HOLD;

	pose = PP_STAND;

	weaponNode->setVisible(true);
	Bone* rightHand = entity->getSkeleton()->getBone("hand_r");
	weaponNode->setPosition(rightHand->_getDerivedPosition());
	weaponNode->setOrientation(rightHand->_getDerivedOrientation());

}

bool PlayerCharacter::hitQuery()
{
	static Ray shootRay;
	
	shootRay.setOrigin(rayOrigin);
	shootRay.setDirection(lookDir);
	raySceneQuery->setRay(shootRay);
	RaySceneQueryResult& queryResult = raySceneQuery->execute();

	//printf("hits: %d ", queryResult.size());
	if (queryResult.size() != 0)
	{
		RaySceneQueryResult::iterator i = queryResult.begin();
		float shootDist = -1;
		while ( i != queryResult.end() )
		{
			//printf("dist=%f;  ",i->distance);
			if (i->distance > 20 && i->distance < 1000) 
			{
				shootDist = i->distance;
				break;
			}
			i++;
		}
		//printf("shootDist=%f\n", shootDist);
		if (shootDist != -1) hud->hits += level->shootTargetAt(rayOrigin + shootDist * lookDir);
	}
	//printf("\n");
	return false;
}

void PlayerCharacter::reload()
{
	if(inAction()) return;

	action = PA_SHOOT;
	shootState = SH_RELOAD;
	hud->reloadMagazine();
	hud->setShowMagazineCounter(false);
}

void PlayerCharacter::shoot(bool b)
{
//	if(inAction()) return;
	if (shootState == SH_RELOAD) return;

	if (action == PA_SHOOT)
	{
		if (b == true && hud->magazine > 0) 
		{
			shootState = SH_SHOOT;
			hud->magazine--;
			hitQuery();
		}
		else
		{
			shootState = SH_AIM;
		}
	}
}

bool PlayerCharacter::takeAim(bool b)
{
//	if(inAction())
//		return;
	if (shootState == SH_RELOAD && action == PA_SHOOT) return false;
	if (b == true) 
	{
		action = PA_SHOOT;
		shootState = SH_TAKE_AIM;
	}
	else 
	{
		action = PA_SHOOT;
		shootState = SH_END_AIM;
	}
	return true;
}

void PlayerCharacter::move(float amount)
{
	if(inAction())
		return;

	//node->translate(amount * Vector3::UNIT_Z, Node::TS_LOCAL);

	Vector3 m = amount * lookDir * Vector3(1,0,1);
	node->translate(m);

	action = PA_WEAPON_HOLD;
	pose = PP_RUN;
}

bool PlayerCharacter::inAction()
{
	return !actionAnim->getLoop() && !actionAnim->hasEnded();
}
