#include "stdafx.h"
#include "PlayerController.h"

using namespace Ogre;

PlayerController::PlayerController(PlayerCharacter* ch, ThirdPersonCamera* cam, Hud* h)
{
	character = ch;
	camera = cam;
	hud = h;
}

PlayerController::~PlayerController(void)
{
}

void PlayerController::update(float t, float dt, OIS::Keyboard* keyboard, OIS::Mouse* mouse)
{
	if (mTimeUntilNextToggle >= 0)
		mTimeUntilNextToggle -= dt;
		
	const OIS::MouseState &ms = mouse->getMouseState();

	if( keyboard->isKeyDown(OIS::KC_W))
			character->move(30 * dt);
	if( keyboard->isKeyDown(OIS::KC_S))
			character->move(30 * -dt);
	//if( keyboard->isKeyDown(OIS::KC_D))
	//		character->turnHorizontal(-dt);
	//if( keyboard->isKeyDown(OIS::KC_A))
	//		character->turnHorizontal(dt);
}

bool PlayerController::keyPressed(const OIS::KeyEvent &keyEventRef)
{
	if (keyEventRef.key == OIS::KC_R)
		character->reload();
	if (keyEventRef.key == OIS::KC_U)
		character->setFollowTruck(false);
	return true;
}

bool PlayerController::mouseMoved(const OIS::MouseEvent &evt)
{
	Quaternion rotX( Degree(evt.state.X.rel * -0.1f), Vector3::UNIT_Y );
	Quaternion rotY( Degree(evt.state.Y.rel * -0.1f), character->getLookDirection().crossProduct( Vector3::UNIT_Y ).normalisedCopy() );
	character->setLookDirection( rotX * rotY * character->getLookDirection() );
	return true;
}

bool PlayerController::mousePressed(const OIS::MouseEvent &evt, OIS::MouseButtonID id)
{
	if(id == OIS::MB_Left)
		character->shoot(true);
	if(id == OIS::MB_Right) {
		character->takeAim(true);
		camera->setAim(true);
		hud->setShowCrosshair(true);	
	}
	return true;
}

bool PlayerController::mouseReleased(const OIS::MouseEvent &evt, OIS::MouseButtonID id)
{
	if(id == OIS::MB_Left)
		character->shoot(false);
	if(id == OIS::MB_Right) {
		character->takeAim(false);
		camera->setAim(false);
		hud->setShowCrosshair(false);
	}
	return true;
}