// OgreLab1.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "Ogre.h"
#include "InputHandler.h"
#include "Level.h"
#include "PlayerCharacter.h"
#include "ThirdPersonCamera.h"
#include "PlayerController.h"

using namespace Ogre;

Root* ogreRoot;
SceneManager* sceneManager;
Camera* camera;
RenderWindow* renderWindow;
InputHandler* inputHandler;
PlayerCharacter* playerCharacter;
ThirdPersonCamera* TPCamera;
PlayerController* playerController;
Hud* hud;
Level* level;

class mainFrameListener : public FrameListener
{
protected:
	float t;
public:
	mainFrameListener()
	{
		t = 0;
	}

	bool frameStarted(const FrameEvent& evt)
	{
		float dt = evt.timeSinceLastFrame;
		t += dt;
		playerCharacter->update(t, dt);
		TPCamera->update(t, dt);
		inputHandler->update(t,dt);
		hud->update(t,dt);
		level->update(t, dt);

		return true;
	}
};

void init()
{
	inputHandler = new InputHandler(renderWindow);
	
	camera = sceneManager->createCamera("Main Camera");       
	camera->setPosition(20, 20, 20); 
	camera->lookAt(0,5,0);
	camera->setFarClipDistance(500);
	camera->setNearClipDistance(1);

	Viewport* viewport = renderWindow->addViewport(camera);
	renderWindow->setActive(true); 
	
	hud = new Hud();
	level = new Level(sceneManager);
	playerCharacter = new PlayerCharacter(sceneManager, hud, level);
	playerCharacter->setTruckNode( level->getTruckNode() );
	playerCharacter->setFollowTruck(true);

	TPCamera = new ThirdPersonCamera();
	TPCamera->setCamera(camera);
	TPCamera->setCharacter(playerCharacter);

	playerController = new PlayerController(playerCharacter, TPCamera, hud);
	inputHandler->setPlayerController(playerController);

	ogreRoot->addFrameListener(new mainFrameListener());
}

int _tmain(int argc, _TCHAR* argv[])
{
	ogreRoot = new Root("","");
#ifdef _DEBUG
	ogreRoot->loadPlugin("RenderSystem_Direct3D9_d");
	ogreRoot->loadPlugin("Plugin_OctreeSceneManager_d");
#else
	ogreRoot->loadPlugin("RenderSystem_Direct3D9");
	ogreRoot->loadPlugin("Plugin_OctreeSceneManager");
#endif

	RenderSystemList* list = ogreRoot->getAvailableRenderers(); 
	ogreRoot->setRenderSystem(list->at(0));
	ogreRoot->initialise(false);
	renderWindow = ogreRoot->createRenderWindow("GameDev Ogre Player Character", 1024, 768, false);

	sceneManager = ogreRoot->createSceneManager("OctreeSceneManager"); 
	
	Light * mainLight0 = sceneManager->createLight("mainLight0");
	mainLight0->setType(Light::LT_DIRECTIONAL);
	mainLight0->setDirection(0.4f, -1, 0);

	//Light * mainLight1 = sceneManager->createLight("mainLight1");
	//mainLight1->setType(Light::LT_DIRECTIONAL);
	//mainLight1->setDirection(0, -1, -0.4f);
	
	sceneManager->setAmbientLight(Ogre::ColourValue(0.7f, 0.7f, 0.7f));

	sceneManager->setShadowTechnique(SHADOWTYPE_STENCIL_ADDITIVE);

	ResourceGroupManager::getSingleton().addResourceLocation("media","FileSystem","General", true);
	ResourceGroupManager::getSingleton().addResourceLocation("media/sky","FileSystem","General", true);
	ResourceGroupManager::getSingleton().addResourceLocation("media/plane","FileSystem","General", true);
	ResourceGroupManager::getSingleton().addResourceLocation("media/character","FileSystem","General", false);
	ResourceGroupManager::getSingleton().addResourceLocation("media/truck","FileSystem","General", false);
	ResourceGroupManager::getSingleton().addResourceLocation("media/hud","FileSystem","General", false);
	ResourceGroupManager::getSingleton().addResourceLocation("media/target","FileSystem","General", false);
	ResourceGroupManager::getSingleton().addResourceLocation("media/crate","FileSystem","General", false);
	//ResourceGroupManager::getSingleton().addResourceLocation("media/character.zip","Zip","General", false);
	ResourceGroupManager::getSingleton().initialiseAllResourceGroups(); 

	
	init();
	
	//ogreRoot->startRendering(); 
	while(true)
	{
		if(renderWindow->isClosed())
		{
			ogreRoot->shutdown();
			break;
		}
		WindowEventUtilities::messagePump();
		
		if(!ogreRoot->renderOneFrame())
		{
			ogreRoot->shutdown();
			break;
		}
	}

	return 0;
}

